﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace CarServiceBusinessLogic.BusinessLogics
{
	public class CustomerLogic : ICustomerLogic
	{
		private readonly ILogger _logger;
		private readonly ICustomerStorage _customerStorage;
		public CustomerLogic(ILogger<CustomerLogic> logger, ICustomerStorage customerStorage)
		{
			_logger = logger;
			_customerStorage = customerStorage;
		}
		public List<CustomerViewModel>? ReadList(CustomerSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _customerStorage.GetFullList() : _customerStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public CustomerViewModel? ReadElement(CustomerSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _customerStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(CustomerBindingModel model)
		{
			CheckModel(model);
			if (_customerStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
	    }
		public bool Update(CustomerBindingModel model)
		{
			CheckModel(model);
			if (_customerStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(CustomerBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_customerStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(CustomerBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("Customer. Id: {Id}", model.Id);
		}
	}
}
