﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace CarServiceBusinessLogic.BusinessLogics
{
	public class RepairRequestLogic : IRepairRequestLogic
	{
		private readonly ILogger _logger;
		private readonly IRepairRequestStorage _repairRequestStorage;
		public RepairRequestLogic(ILogger<RepairRequestLogic> logger, IRepairRequestStorage repairRequestStorage)
		{
			_logger = logger;
			_repairRequestStorage = repairRequestStorage;
		}
		public List<RepairRequestViewModel>? ReadList(RepairRequestSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _repairRequestStorage.GetFullList() : _repairRequestStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public RepairRequestViewModel? ReadElement(RepairRequestSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _repairRequestStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(RepairRequestBindingModel model)
		{
			CheckModel(model);
			if (_repairRequestStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(RepairRequestBindingModel model)
		{
			CheckModel(model);
			if (_repairRequestStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(RepairRequestBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_repairRequestStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(RepairRequestBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("RepairRequest. Id: {Id}", model.Id);
		}
	}
}
