﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace CarServiceBusinessLogic.BusinessLogics
{
	public class VehicleLogic : IVehicleLogic
	{
		private readonly ILogger _logger;
		private readonly IVehicleStorage _vehicleStorage;
		public VehicleLogic(ILogger<VehicleLogic> logger, IVehicleStorage vehicleStorage)
		{
			_logger = logger;
			_vehicleStorage = vehicleStorage;
		}
		public List<VehicleViewModel>? ReadList(VehicleSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _vehicleStorage.GetFullList() : _vehicleStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public VehicleViewModel? ReadElement(VehicleSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _vehicleStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(VehicleBindingModel model)
		{
			CheckModel(model);
			if (_vehicleStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(VehicleBindingModel model)
		{
			CheckModel(model);
			if (_vehicleStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(VehicleBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_vehicleStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(VehicleBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("Vehicle. Id: {Id}", model.Id);
		}
	}
}
