﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace CarServiceBusinessLogic.BusinessLogics
{
	public class WorkInRequestLogic : IWorkInRequestLogic
	{
		private readonly ILogger _logger;
		private readonly IWorkInRequestStorage _workInRequestStorage;
		public WorkInRequestLogic(ILogger<WorkInRequestLogic> logger, IWorkInRequestStorage workInRequestStorage)
		{
			_logger = logger;
			_workInRequestStorage = workInRequestStorage;
		}
		public List<WorkInRequestViewModel>? ReadList(WorkInRequestSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _workInRequestStorage.GetFullList() : _workInRequestStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public WorkInRequestViewModel? ReadElement(WorkInRequestSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _workInRequestStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(WorkInRequestBindingModel model)
		{
			CheckModel(model);
			if (_workInRequestStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(WorkInRequestBindingModel model)
		{
			CheckModel(model);
			if (_workInRequestStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(WorkInRequestBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_workInRequestStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(WorkInRequestBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("WorkInRequest. Id: {Id}", model.Id);
		}
	}
}
