﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace CarServiceBusinessLogic.BusinessLogics
{
	public class WorkPaymentLogic : IWorkPaymentLogic
	{
		private readonly ILogger _logger;
		private readonly IWorkPaymentStorage _workPaymentStorage;
		public WorkPaymentLogic(ILogger<WorkPaymentLogic> logger, IWorkPaymentStorage workPaymentStorage)
		{
			_logger = logger;
			_workPaymentStorage = workPaymentStorage;
		}
		public List<WorkPaymentViewModel>? ReadList(WorkPaymentSearchModel? model)
		{
			_logger.LogInformation("ReadList. Id: {Id}", model?.Id);
			var list = model == null ? _workPaymentStorage.GetFullList() : _workPaymentStorage.GetFilteredList(model);
			if (list == null)
			{
				_logger.LogWarning("ReadList return null list");
				return null;
			}
			_logger.LogInformation("ReadList. Count: {Count}", list.Count);
			return list;
		}
		public WorkPaymentViewModel? ReadElement(WorkPaymentSearchModel model)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			_logger.LogInformation("ReadElement. Id: {Id}", model.Id);
			var element = _workPaymentStorage.GetElement(model);
			if (element == null)
			{
				_logger.LogWarning("ReadElement element not found");
				return null;
			}
			_logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
			return element;
		}
		public bool Create(WorkPaymentBindingModel model)
		{
			CheckModel(model);
			if (_workPaymentStorage.Insert(model) == null)
			{
				_logger.LogWarning("Insert operation failed");
				return false;
			}
			return true;
		}
		public bool Update(WorkPaymentBindingModel model)
		{
			CheckModel(model);
			if (_workPaymentStorage.Update(model) == null)
			{
				_logger.LogWarning("Update operation failed");
				return false;
			}
			return true;
		}
		public bool Delete(WorkPaymentBindingModel model)
		{
			CheckModel(model, false);
			_logger.LogInformation("Delete. Id:{Id}", model.Id);
			if (_workPaymentStorage.Delete(model) == null)
			{
				_logger.LogWarning("Delete operation failed");
				return false;
			}
			return true;
		}
		private void CheckModel(WorkPaymentBindingModel model, bool withParams = true)
		{
			if (model == null)
			{
				throw new ArgumentNullException(nameof(model));
			}
			if (!withParams)
			{
				return;
			}
			_logger.LogInformation("WorkPayment. Id: {Id}", model.Id);
		}
	}
}
