﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using CarServiceDatabase.Models;
using Microsoft.EntityFrameworkCore;

namespace CarServiceDatabase.Implements
{
	public class ItemForRepairStorage : IItemForRepairStorage
	{
		public List<ItemForRepairViewModel> GetFullList()
		{
			using var context = new CarServiceDbContext();
			return context.ItemsForRepair
				.Include(x => x.Item)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public List<ItemForRepairViewModel> GetFilteredList(ItemForRepairSearchModel model)
		{
			using var context = new CarServiceDbContext();
			return context.ItemsForRepair
				.Where(x => x.RepairRequestId == model.Id)
				.Include(x => x.Item)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public ItemForRepairViewModel? GetElement(ItemForRepairSearchModel model)
		{
			if (model == null)
			{
				return null;
			}
			using var context = new CarServiceDbContext();
			if (model.Id.HasValue)
			{
				return context.ItemsForRepair
					.Include(x => x.Item)
					.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
			}
			if (model.ItemId.HasValue)
			{
				return context.ItemsForRepair
					.Include(x => x.Item)
					.FirstOrDefault(x => x.ItemId == model.ItemId)?.GetViewModel;
			}
			return null;
		}
		public ItemForRepairViewModel? Insert(ItemForRepairBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var newItemForRepair = ItemForRepair.Create(context, model);
			if (newItemForRepair != null)
			{
				context.ItemsForRepair.Add(newItemForRepair);
				context.SaveChanges();
				return newItemForRepair.GetViewModel;
			}
			return null;
		}
		public ItemForRepairViewModel? Update(ItemForRepairBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var itemForRepair = context.ItemsForRepair.FirstOrDefault(x => x.Id == model.Id);
			if (itemForRepair == null)
			{
				return null;
			}
			itemForRepair.Update(context, model);
			context.SaveChanges();
			return itemForRepair.GetViewModel;
		}
		public ItemForRepairViewModel? Delete(ItemForRepairBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var itemForRepair = context.ItemsForRepair.FirstOrDefault(x => x.Id == model.Id);
			if (itemForRepair == null)
			{
				return null;
			}
			context.ItemsForRepair.Remove(itemForRepair);
			context.SaveChanges();
			return itemForRepair.GetViewModel;
		}
	}
}
