﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using CarServiceDatabase.Models;
using Microsoft.EntityFrameworkCore;

namespace CarServiceDatabase.Implements
{
	public class WorkInRequestStorage : IWorkInRequestStorage
	{
		public List<WorkInRequestViewModel> GetFullList()
		{
			using var context = new CarServiceDbContext();
			return context.WorksInRequest
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public List<WorkInRequestViewModel> GetFilteredList(WorkInRequestSearchModel model)
		{
			using var context = new CarServiceDbContext();
			return context.WorksInRequest
				.Include(x => x.Work)
				.Where(x => x.RepairRequestId == model.Id)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public WorkInRequestViewModel? GetElement(WorkInRequestSearchModel model)
		{
			if (model == null)
			{
				return null;
			}
			using var context = new CarServiceDbContext();
			if (model.Id.HasValue)
			{
				return context.WorksInRequest
					.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
			}
			return null;
		}
		public WorkInRequestViewModel? Insert(WorkInRequestBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var newWorkInRequest = WorkInRequest.Create(context, model);
			if (newWorkInRequest != null)
			{
				context.WorksInRequest.Add(newWorkInRequest);
				context.SaveChanges();
				return newWorkInRequest.GetViewModel;
			}
			return null;
		}
		public WorkInRequestViewModel? Update(WorkInRequestBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var workInRequest = context.WorksInRequest.FirstOrDefault(x => x.Id == model.Id);
			if (workInRequest == null)
			{
				return null;
			}
			workInRequest.Update(context, model);
			context.SaveChanges();
			return workInRequest.GetViewModel;
		}
		public WorkInRequestViewModel? Delete(WorkInRequestBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var workInRequest = context.WorksInRequest.FirstOrDefault(x => x.Id == model.Id);
			if (workInRequest == null)
			{
				return null;
			}
			context.WorksInRequest.Remove(workInRequest);
			context.SaveChanges();
			return workInRequest.GetViewModel;
		}
	}
}
