﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using CarServiceDatabase.Models;
using Microsoft.EntityFrameworkCore;

namespace CarServiceDatabase.Implements
{
	public class WorkPaymentStorage : IWorkPaymentStorage
	{
		public List<WorkPaymentViewModel> GetFullList()
		{
			using var context = new CarServiceDbContext();
			return context.WorkPayments
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public List<WorkPaymentViewModel> GetFilteredList(WorkPaymentSearchModel model)
		{
			using var context = new CarServiceDbContext();
			return context.WorkPayments
				.Where(x => x.Id == model.Id)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public WorkPaymentViewModel? GetElement(WorkPaymentSearchModel model)
		{
			if (model == null)
			{
				return null;
			}
			using var context = new CarServiceDbContext();
			if (model.Id.HasValue)
			{
				return context.WorkPayments
					.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
			}
			return null;
		}
		public List<ReportWorksWithPaymentsViewModel> GetPaymentsByWorks (WorkPaymentSearchModel model)
		{
			if (model.DateFrom == null || model.DateTo == null)
			{
				return new();
			}
			using var context = new CarServiceDbContext();
			return context.WorkPayments
				.Include(wp => wp.WorkInRequest)
				.ThenInclude(wir => wir.RepairRequest)
				.ThenInclude(rr => rr.Vehicle)
				.ThenInclude(v => v.Customer)
				.Include(wp => wp.WorkInRequest)
				.ThenInclude(wp => wp.Work)
				.Where(wp => wp.DatePayment >= model.DateFrom && wp.DatePayment <= model.DateTo)
				.Select(wp => new ReportWorksWithPaymentsViewModel()
				{
					PaymentDate = wp.DatePayment,
					CustomerName = wp.WorkInRequest.RepairRequest.Vehicle.Customer.Name + " " + wp.WorkInRequest.RepairRequest.Vehicle.Customer.Surname,
					RepairRequestId = wp.WorkInRequest.RepairRequest.Id,
					VehicleNameAndPlate = wp.WorkInRequest.RepairRequest.Vehicle.Name + " " + wp.WorkInRequest.RepairRequest.Vehicle.Plate,
					WorkName = wp.WorkInRequest.Work.Name,
					Count = wp.WorkInRequest.Count,
					WorkPrice = wp.WorkInRequest.Work.Price,
					TotalSum = Convert.ToDecimal(wp.WorkInRequest.Count * wp.WorkInRequest.Work.Price),
					PaymentSum = wp.Sum,
					Paid = context.WorkPayments.Where(x => x.WorkInRequestId == wp.WorkInRequestId).Select(x => x.Sum).Sum(),
					NotPaid = Convert.ToDecimal(wp.WorkInRequest.Count * wp.WorkInRequest.Work.Price - context.WorkPayments.Where(x => x.WorkInRequestId == wp.WorkInRequestId).Select(x => x.Sum).Sum())
				})
				.ToList();
		}
		public WorkPaymentViewModel? Insert(WorkPaymentBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var newWorkPayment = WorkPayment.Create(context, model);
			if (newWorkPayment != null)
			{
				context.WorkPayments.Add(newWorkPayment);
				context.SaveChanges();
				return newWorkPayment.GetViewModel;
			}
			return null;
		}
		public WorkPaymentViewModel? Update(WorkPaymentBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var workPayment = context.WorkPayments.FirstOrDefault(x => x.Id == model.Id);
			if (workPayment == null)
			{
				return null;
			}
			workPayment.Update(context, model);
			context.SaveChanges();
			return workPayment.GetViewModel;
		}
		public WorkPaymentViewModel? Delete(WorkPaymentBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var workPayment = context.WorkPayments.FirstOrDefault(x => x.Id == model.Id);
			if (workPayment == null)
			{
				return null;
			}
			context.WorkPayments.Remove(workPayment);
			context.SaveChanges();
			return workPayment.GetViewModel;
		}
	}
}
