﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using CarServiceDatabase.Models;
using Microsoft.EntityFrameworkCore;

namespace CarServiceDatabase.Implements
{
	public class WorkStorage : IWorkStorage
	{
		public List<WorkViewModel> GetFullList()
		{
			using var context = new CarServiceDbContext();
			return context.Works
				.Include(x => x.Worker)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public List<WorkViewModel> GetFilteredList(WorkSearchModel model)
		{
			using var context = new CarServiceDbContext();
			return context.Works
				.Where(x => x.WorkerId == model.Id)
				.Include(x => x.Worker)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public WorkViewModel? GetElement(WorkSearchModel model)
		{
			if (model == null)
			{
				return null;
			}
			using var context = new CarServiceDbContext();
			if (model.Id.HasValue)//сначала ищем по Id
			{
				return context.Works
					.Include(x => x.Worker)
					.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
			}
			if (!string.IsNullOrEmpty(model.Name))//затем по названию
			{
				return context.Works
					.Include(x => x.Worker)
					.FirstOrDefault(x => x.Name == model.Name)?.GetViewModel;
			}
			return null;
		}
		public List<ReportWorkWithRequestsViewModel> GetWorksWithRequest(WorkSearchModel model)
		{
			if (model.SelectedWorksIds == null)
			{
				return new();
			}
			using var context = new CarServiceDbContext();
			return context.Works
				.Where(w => model.SelectedWorksIds.Contains(w.Id))
				.Select(w => new ReportWorkWithRequestsViewModel()
				{
					WorkName = w.Name,
					RepairRequests = context.WorksInRequest
						.Include(wir => wir.RepairRequest)
						.ThenInclude(rr => rr.Vehicle)
						.ThenInclude(v => v.Customer)
						.Where(wir => wir.WorkId == w.Id)
						.Select(wir => new ReportRepairRequestViewModel()
						{
							RepairRequestId = wir.RepairRequest.Id,
							RepairRequestDateCreated = wir.RepairRequest.DateCreated,
							CustomerName = wir.RepairRequest.Vehicle.Customer.Name,
							VehicleName = wir.RepairRequest.Vehicle.Name,
							Plate = wir.RepairRequest.Vehicle.Plate ?? "[отсутствует]",
							WorksCount = wir.Count
						})
						.ToList()
				})
				.ToList();
		}
		public WorkViewModel? Insert(WorkBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var newWork = Work.Create(context, model);
			if (newWork != null)
			{
				context.Works.Add(newWork);
				context.SaveChanges();
				return newWork.GetViewModel;
			}
			return null;
		}
		public WorkViewModel? Update(WorkBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var work = context.Works.FirstOrDefault(x => x.Id == model.Id);
			if (work == null)
			{
				return null;
			}
			work.Update(context, model);
			context.SaveChanges();
			return work.GetViewModel;
		}
		public WorkViewModel? Delete(WorkBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var work = context.Works.FirstOrDefault(x => x.Id == model.Id);
			if (work == null)
			{
				return null;
			}
			context.Works.Remove(work);
			context.SaveChanges();
			return work.GetViewModel;
		}
	}
}