﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.SearchModels;
using CarServiceContracts.StorageContracts;
using CarServiceContracts.ViewModels;
using CarServiceDatabase.Models;

namespace CarServiceDatabase.Implements
{
	public class WorkerStorage : IWorkerStorage
	{
		public List<WorkerViewModel> GetFullList()
		{
			using var context = new CarServiceDbContext();
			return context.Workers
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public List<WorkerViewModel> GetFilteredList(WorkerSearchModel model)
		{
			using var context = new CarServiceDbContext();
			return context.Workers
				.Where(x => x.Id == model.Id)
				.Select(x => x.GetViewModel)
				.ToList();
		}
		public WorkerViewModel? GetElement(WorkerSearchModel model)
		{
			if (model == null)
			{
				return null;
			}
			using var context = new CarServiceDbContext();
			if (model.Id.HasValue)//Сначала ищем по Id
			{
				return context.Workers.FirstOrDefault(x => x.Id == model.Id)?.GetViewModel;
			}
			else if (!string.IsNullOrEmpty(model.Login) && string.IsNullOrEmpty(model.Password))//Затем по логину (для проверки уникальности)
			{
				return context.Workers
					.FirstOrDefault(x => x.Login == model.Login)?.GetViewModel;
			}
			else if (!string.IsNullOrEmpty(model.Login) && !string.IsNullOrEmpty(model.Password))//Затем по логину и паролю (для входа)
			{
				return context.Workers
					.FirstOrDefault(x => x.Login == model.Login && x.Password == model.Password)?.GetViewModel;
			}
			return null;
		}
		public WorkerViewModel? Insert(WorkerBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var newWorker = Worker.Create(model);
			if (newWorker != null)
			{
				context.Workers.Add(newWorker);
				context.SaveChanges();
				return newWorker.GetViewModel;
			}
			return null;
		}
		public WorkerViewModel? Update(WorkerBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var worker = context.Workers
				.FirstOrDefault(x => x.Id == model.Id);
			if (worker == null)
			{
				return null;
			}
			worker.Update(model);
			context.SaveChanges();
			return worker.GetViewModel;
		}
		public WorkerViewModel? Delete(WorkerBindingModel model)
		{
			using var context = new CarServiceDbContext();
			var worker = context.Workers
				.FirstOrDefault(x => x.Id == model.Id);
			if (worker == null)
			{
				return null;
			}
			context.Workers.Remove(worker);
			context.SaveChanges();
			return worker.GetViewModel;
		}
	}
}
