﻿using CarServiceContracts.BindingModels;
using CarServiceContracts.Models;
using CarServiceContracts.ViewModels;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace CarServiceDatabase.Models
{
	public class Work : IWorkModel
	{
		public int Id { get; private set; }
		[Required]
		public string Name { get; private set; } = string.Empty;
		[Required, Column(TypeName = "decimal (10,2)")]
		public decimal Price { get; private set; }
		[Required, Column(TypeName = "decimal (3,1)")]
		public decimal Duration { get; private set; }
		[Required]
		public int WorkerId { get; private set; }
		/// <summary>
		/// Работы в заявке
		/// </summary>
		[ForeignKey("WorkId")]
		public virtual List<WorkInRequest> WorksInRequest { get; set; } = new();
		public virtual Worker Worker { get; set; } = new();
		public static Work? Create(CarServiceDbContext context, WorkBindingModel? model)
		{
			if (model == null)
			{
				return null;
			}
			return new()
			{
				Name = model.Name,
				Price = model.Price,
				Duration = model.Duration,
				Worker = context.Workers.First(x => x.Id == model.WorkerId)
			};
		}
		public void Update(CarServiceDbContext context, WorkBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			Name = model.Name;
			Price = model.Price;
			Duration = model.Duration;
			Worker = context.Workers.First(x => x.Id == model.WorkerId);
		}
		public WorkViewModel GetViewModel => new()
		{
			Id = Id,
			Name = Name,
			Price = Price,
			Duration = Duration,
			WorkerId = WorkerId,
			WorkerName = Worker.Name + " " + Worker.Surname
		};
	}
}
