using CarServiceBusinessLogic.BusinessLogics;
using CarServiceContracts.BusinessLogicsContracts;
using CarServiceContracts.StorageContracts;
using CarServiceDatabase.Implements;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;

namespace CarServiceView
{
	internal static class Program
	{
		private static ServiceProvider? _serviceProvider;
		public static ServiceProvider? ServiceProvider => _serviceProvider;
		[STAThread]
		static void Main()
		{
			ApplicationConfiguration.Initialize();
			var services = new ServiceCollection();
			ConfigureServices(services);
			_serviceProvider = services.BuildServiceProvider();
			var _reportLogic = _serviceProvider.GetRequiredService<IReportLogic>();
			var list = _reportLogic.GetRequestsByWorks(new() { SelectedWorks = new() { 1 } });
			Application.Run(_serviceProvider.GetRequiredService<FormItemForRepairTest>());
		}
		private static void ConfigureServices(ServiceCollection services)
		{
			services.AddLogging(option =>
			{
				option.SetMinimumLevel(LogLevel.Information);
				option.AddNLog("nlog.config");
			});
			services.AddTransient<ICustomerLogic, CustomerLogic>();
			services.AddTransient<IItemForRepairLogic, ItemForRepairLogic>();
			services.AddTransient<IItemLogic, ItemLogic>();
			services.AddTransient<IRepairRequestLogic, RepairRequestLogic>();
			services.AddTransient<IVehicleLogic, VehicleLogic>();
			services.AddTransient<IWorkerLogic, WorkerLogic>();
			services.AddTransient<IWorkInRequestLogic, WorkInRequestLogic>();
			services.AddTransient<IWorkLogic, WorkLogic>();
			services.AddTransient<IWorkPaymentLogic, WorkPaymentLogic>();
			services.AddTransient<IReportLogic, ReportLogic>();

			services.AddTransient<IWorkerStorage, WorkerStorage>();
			services.AddTransient<IWorkStorage, WorkStorage>();
			services.AddTransient<IItemStorage, ItemStorage>();
			services.AddTransient<IItemForRepairStorage, ItemForRepairStorage>();
			services.AddTransient<IRepairRequestStorage, RepairRequestStorage>();

			services.AddTransient<FormRegistrationWorker>();
			services.AddTransient<FormWorkTest>();
			services.AddTransient<FormAddWorkTest>();
			services.AddTransient<FormItemTest>();
			services.AddTransient<FormAddItemTest>();
			services.AddTransient<FormItemForRepairTest>();
			services.AddTransient<FormAddItemForRepairTest>();
			services.AddTransient<FormUpdateItemForRepairTest>();
		}
	}
}